import * as fs from 'node:fs';
export interface OpenFileOptions {
    /**
     * Overrides the name of the file. Default is the name of the file on disk.
     */
    name?: string;
    /**
     * Overrides the MIME type of the file. Default is determined by the file extension.
     */
    type?: string;
    /**
     * Overrides the last modified timestamp of the file. Default is the file's last modified time.
     */
    lastModified?: number;
}
/**
 * Returns a `File` from the local filesytem.
 *
 * [MDN `File` Reference](https://developer.mozilla.org/en-US/docs/Web/API/File)
 *
 * @param filename The path to the file
 * @param options Options to override the file's metadata
 * @returns A `File` object
 */
export declare function openFile(filename: string, options?: OpenFileOptions): File;
export { type OpenFileOptions as GetFileOptions, openFile as getFile };
/**
 * Writes a `File` to the local filesytem and resolves when the stream is finished.
 *
 * [MDN `File` Reference](https://developer.mozilla.org/en-US/docs/Web/API/File)
 *
 * @param to The path to write the file to, or an open file descriptor
 * @param file The file to write
 * @returns A promise that resolves when the file is written
 */
export declare function writeFile(to: string | number | fs.promises.FileHandle, file: File): Promise<void>;
//# sourceMappingURL=fs.d.ts.map