"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/lazy-file.ts
var lazy_file_exports = {};
__export(lazy_file_exports, {
  LazyBlob: () => LazyBlob,
  LazyFile: () => LazyFile,
  getByteLength: () => getByteLength,
  getIndexes: () => getIndexes
});
module.exports = __toCommonJS(lazy_file_exports);

// src/lib/byte-range.ts
function getByteLength(range, size) {
  let [start, end] = getIndexes(range, size);
  return end - start;
}
function getIndexes(range, size) {
  let start = Math.min(Math.max(0, range.start < 0 ? size + range.start : range.start), size);
  let end = Math.min(Math.max(start, range.end < 0 ? size + range.end : range.end), size);
  return [start, end];
}

// src/lib/lazy-file.ts
var LazyBlob = class extends Blob {
  #content;
  constructor(parts, options) {
    super([], options);
    this.#content = new BlobContent(parts, options);
  }
  /**
   * Returns the blob's contents as an [`ArrayBuffer`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/ArrayBuffer).
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/arrayBuffer)
   */
  async arrayBuffer() {
    return this.#content.arrayBuffer();
  }
  /**
   * Returns the blob's contents as a byte array.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/bytes)
   */
  async bytes() {
    return this.#content.bytes();
  }
  /**
   * The size of the blob in bytes.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/size)
   */
  get size() {
    return this.#content.size;
  }
  /**
   * Returns a new [`Blob`](https://developer.mozilla.org/en-US/docs/Web/API/Blob) that contains the data in the specified range.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/slice)
   */
  slice(start, end, contentType) {
    return this.#content.slice(start, end, contentType);
  }
  /**
   * Returns a stream that can be used to read the blob's contents.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/stream)
   */
  stream() {
    return this.#content.stream();
  }
  /**
   * Returns the blob's contents as a string.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/text)
   */
  async text() {
    return this.#content.text();
  }
};
var LazyFile = class extends File {
  #content;
  constructor(parts, name, options) {
    super([], name, options);
    this.#content = new BlobContent(parts, options);
  }
  /**
   * Returns the file's content as an [`ArrayBuffer`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/ArrayBuffer).
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/arrayBuffer)
   */
  async arrayBuffer() {
    return this.#content.arrayBuffer();
  }
  /**
   * Returns the file's contents as a byte array.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/bytes)
   */
  async bytes() {
    return this.#content.bytes();
  }
  /**
   * The size of the file in bytes.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/size)
   */
  get size() {
    return this.#content.size;
  }
  /**
   * Returns a new [`Blob`](https://developer.mozilla.org/en-US/docs/Web/API/Blob) that contains the data in the specified range.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/slice)
   */
  slice(start, end, contentType) {
    return this.#content.slice(start, end, contentType);
  }
  /**
   * Returns a stream that can be used to read the file's contents.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/stream)
   */
  stream() {
    return this.#content.stream();
  }
  /**
   * Returns the file's contents as a string.
   *
   * [MDN Reference](https://developer.mozilla.org/en-US/docs/Web/API/Blob/text)
   */
  async text() {
    return this.#content.text();
  }
};
var BlobContent = class {
  source;
  totalSize;
  range;
  type;
  constructor(parts, options) {
    if (Array.isArray(parts)) {
      this.source = [];
      this.totalSize = 0;
      for (let part of parts) {
        if (part instanceof Blob) {
          this.source.push(part);
          this.totalSize += part.size;
        } else {
          let array;
          if (typeof part === "string") {
            array = new TextEncoder().encode(part);
          } else if (ArrayBuffer.isView(part)) {
            array = new Uint8Array(part.buffer, part.byteOffset, part.byteLength);
          } else {
            array = new Uint8Array(part);
          }
          this.source.push(array);
          this.totalSize += array.byteLength;
        }
      }
    } else {
      this.source = parts;
      this.totalSize = parts.byteLength;
    }
    this.range = options?.range;
    this.type = options?.type ?? "";
  }
  async arrayBuffer() {
    return (await this.bytes()).buffer;
  }
  async bytes() {
    let result = new Uint8Array(this.size);
    let offset = 0;
    for await (let chunk of this.stream()) {
      result.set(chunk, offset);
      offset += chunk.length;
    }
    return result;
  }
  get size() {
    return this.range != null ? getByteLength(this.range, this.totalSize) : this.totalSize;
  }
  slice(start = 0, end, contentType = "") {
    let range = this.range != null ? (
      // file.slice().slice() is additive
      { start: this.range.start + start, end: this.range.end + (end ?? 0) }
    ) : { start, end: end ?? this.size };
    return new LazyBlob(this.source, { range, type: contentType });
  }
  stream() {
    if (this.range != null) {
      let [start, end] = getIndexes(this.range, this.totalSize);
      return Array.isArray(this.source) ? streamContentArray(this.source, start, end) : this.source.stream(start, end);
    }
    return Array.isArray(this.source) ? streamContentArray(this.source) : this.source.stream();
  }
  async text() {
    return new TextDecoder("utf-8").decode(await this.bytes());
  }
};
function streamContentArray(content, start = 0, end = Infinity) {
  let index = 0;
  let bytesRead = 0;
  return new ReadableStream({
    async pull(controller) {
      if (index >= content.length) {
        controller.close();
        return;
      }
      let hasPushed = false;
      function pushChunk(chunk) {
        let chunkLength = chunk.byteLength;
        if (!(bytesRead + chunkLength < start || bytesRead >= end)) {
          let startIndex = Math.max(start - bytesRead, 0);
          let endIndex = Math.min(end - bytesRead, chunkLength);
          controller.enqueue(chunk.subarray(startIndex, endIndex));
          hasPushed = true;
        }
        bytesRead += chunkLength;
      }
      async function pushPart(part) {
        if (part instanceof Blob) {
          if (bytesRead + part.size <= start) {
            bytesRead += part.size;
            return;
          }
          for await (let chunk of part.stream()) {
            pushChunk(chunk);
            if (bytesRead >= end) {
              break;
            }
          }
        } else {
          pushChunk(part);
        }
      }
      while (!hasPushed && index < content.length) {
        await pushPart(content[index++]);
        if (bytesRead >= end) {
          controller.close();
          break;
        }
      }
    }
  });
}
//# sourceMappingURL=lazy-file.cjs.map
